# $Id: PathHandler.py,v 1.4 2000/11/07 16:10:40 chrisw Exp $
__doc__ = """Path Handler"""

__version__='$Revision: 1.4 $'[11:-2]

from Globals import HTMLFile      # fakes a method from a DTML file

from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from copy import copy
import Globals
import string

manage_addPathHandlerForm = HTMLFile('www/PathHandlerAdd', globals()) 

def manage_addPathHandler(self, id, method_name, title='', REQUEST=None):
    """Add a Path Handler."""
    self._setObject(id, PathHandler(id, method_name, title))
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

class PathHandler(
                   SimpleItem,           # a Path Handler is a simple object
                   PropertyManager       # ...with properties
                 ): 

    meta_type = 'Path Handler'
    
    manage_options=(
        PropertyManager.manage_options+
        SimpleItem.manage_options
        )
    
    def __init__(self, id, method_name, title=''): 
        """initialise a new instance of Path Handler"""
        self.id = id
        self.title = title

        # The name of the handler method to call
        self.Method = method_name
        # The list of names to process as normal
        self.NamesToProcessNormally=[
            'manage_workspace',
            'manage_propertiesForm',
            'manage_UndoForm',
            'manage_owner',
            'manage_access',
            'manage_UndoForm',
            'manage_editProperties',
            'manage_addProperty',
            'manage_delProperties',
            'manage_undo_transactions',
            'manage_main',
            'manage_changeOwnershipType',
            'manage_changePermissions',
            'manage_listLocalRoles',
            'manage_delLocalroles',
            'manage_setLocalRoles',            
            ]

        # Make our variables available as properties
        properties = list(self._properties)
        properties.append({'id':'Method', 'type': 'string', 'mode': 'w'})
        properties.append({'id':'NamesToProcessNormally', 'type': 'lines', 'mode': 'w'})
        self._properties = tuple(properties)

    def __bobo_traverse__(self, TraversalRequest, name):
        # If this is the first time this __bob_traverse__ method has been called
        # in handling this traversal request, store the path_to_handle

        
        if not TraversalRequest.has_key('path_to_handle'):
            # Make a reversed copy of the TraversalRequestNameStack
            TraversalRequestNameStackReversed=copy(TraversalRequest['TraversalRequestNameStack'])
            TraversalRequestNameStackReversed.reverse()

            # Set path_to_handle in the TraversalRequest.
            TraversalRequest['path_to_handle']=[name]+TraversalRequestNameStackReversed

        # If the name is in the list to exclude, call it.
        if name in self.NamesToProcessNormally:
            return getattr(self,name)

        # otherwise do some 'magic'
        else:
            # If there's no more names left to handle, return the path handling method
            # to the traversal machinery so it gets called next
            if len(TraversalRequest['TraversalRequestNameStack'])==0:
                return getattr(self,self.Method)
            # If there are more names to handle, return ourself, so this __bobo_traverse__ gets
            # called again and we get to ignore the other names.
            else:
                return self

    # This handles the special case where the Path Handler's id is the last one in the
    # traversal request.
    # eg: when the URL ends with the id of the Path Handler.
    def __call__(self, REQUEST={}, **kw):
        """"""
        REQUEST['path_to_handle']=[]
        return getattr(self,self.Method)(self,REQUEST)

    index_html=None

Globals.default__class_init__(PathHandler)
    
constructors = (
  ('manage_addPathHandlerForm', HTMLFile('www/PathHandlerAdd', globals())),
  ('manage_addPathHandler', manage_addPathHandler),
)

    





